/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
  See OTBCopyright.txt for details.


  This software is distributed WITHOUT ANY WARRANTY; without even
  the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
  PURPOSE.  See the above copyright notices for more information.

=========================================================================*/

#include "otbFusionOGRTileFilter.h"
#include "otbImage.h"
#include "otbImageFileReader.h"
#include "itksys/SystemTools.hxx"
#include "otbOGRDataSourceWrapper.h"

int otbFusionOGRTileFilter(int argc, char * argv[])
{
  if (argc != 6)
    {
      std::cerr << "Usage: " << argv[0];
      std::cerr << " inputImage inputOGR outputOGR layerName streamingSize" << std::endl;
      return EXIT_FAILURE;
    }
  
  const char * infname      = argv[1];
  const char * inOGRfname   = argv[2];
  const char * tmpOGRfname  = argv[3];
  const char * layerName    = argv[4];
  unsigned int size         = atoi(argv[5]);
  
  /** Typedefs */
  const unsigned int Dimension = 2;
  typedef float PixelType;
  typedef otb::Image<PixelType, Dimension> ImageType;

  typedef otb::FusionOGRTileFilter<ImageType>   FilterType;
  typedef otb::ImageFileReader<ImageType>       ReaderType;
  
  ReaderType::Pointer reader = ReaderType::New();
  FilterType::Pointer filter = FilterType::New();
  
  //first copy the input OGR file as it will be updated with the fusionned polygons
  itksys::SystemTools::CopyAFile(inOGRfname,tmpOGRfname,true);
  
  
  reader->SetFileName(infname);
  reader->UpdateOutputInformation();
  
  ImageType::SizeType streamSize;
  streamSize.Fill(size);
  
  otb::ogr::DataSource::Pointer ogrDS = otb::ogr::DataSource::New(tmpOGRfname, otb::ogr::DataSource::Modes::write);
  
  filter->SetInput(reader->GetOutput());
  filter->SetOGRDataSource(ogrDS);
  filter->SetStreamSize(streamSize);
  filter->SetLayerName(layerName);
  filter->GenerateData();

  return EXIT_SUCCESS;
}
